# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from collections import OrderedDict

from django.db.models import Q

from kinder.core.dict.models import DouType
from kinder.core.dict.models import UnitKind
from kinder.core.emie_unit.models import EmieUnitModel
from kinder.core.emie_unit.models import UnitDocFiles
from kinder.core.group.models import Group
from kinder.core.group.models import GroupStatusEnum
from kinder.core.group.models import Pupil
from kinder.core.unit.models import FilialData
from kinder.core.unit.models import FilialType
from kinder.core.unit.models import StructureTypes
from kinder.core.unit.models import UnitStatus
from kinder.core.unit.models import Unit
from kinder.core.employer.models import EmployerPost

from .utils import UnitHelper
from .utils import get_file_url
from .utils import try_date_in_string


website_pattern = re.compile("((https?)\:\/\/)?(\S+)")
phone_pattern = re.compile("([0-9]{10}(,[0-9]{10})*)")
email_pattern = re.compile("(\S+@\S+\.\S+)")


ORGANIZATION_TYPES = {
    DouType.MUNICIPAL: 1,
    DouType.GOVERNMENT: 2,
    DouType.NOT_GOVERNMENT_WITH_LICENSE: 3,
    DouType.NOT_GOVERNMENT_WITHOUT_LICENSE: 4,
    DouType.IP_WITH_LICENSE: 5,
    DouType.IP_WITHOUT_LICENSE: 6,
    DouType.DEPARTMENTAL: 7
}


def get_dou_type(dou):

    dou_type = dou.dou_type
    if dou_type is None or dou_type.code == DouType.OTHER:
        code = DouType.GOVERNMENT
    else:
        code = dou_type.code
    return ORGANIZATION_TYPES[code]


# Коды статусов в Информике (отличаются от наших)
WORK = 1
RECONSTRUCTION = 3
IN_CAPITAL_REPAIR = 2
SUSPENDED = 4
NO_CONTINGENT = 5
PENDING_OPEN = 6

# Маппинг этих статусов на наши
STATUSES = (
     (UnitStatus.NONE, WORK),
     (UnitStatus.RECONSTRUCTION, RECONSTRUCTION),
     (UnitStatus.IN_CAPITAL_REPAIR, IN_CAPITAL_REPAIR),
     (UnitStatus.SUSPENDED, SUSPENDED),
     (UnitStatus.MISSING_CONTINGENT, NO_CONTINGENT),
     (UnitStatus.PENDING_OPEN, PENDING_OPEN)
)


def _get_gisdo_unit_status_code(original_code, default=None):
    """Маппинг кодов статусов учреждения для фед. отчётности."""
    return dict(STATUSES).get(original_code, None or default)


# статусы для атрибута `status_building`
FUNCTIONING = 1
NOT_FUNCTIONING = 0

FUNCTIONING_MAPPING = {
    WORK: FUNCTIONING,
    RECONSTRUCTION: NOT_FUNCTIONING,
    IN_CAPITAL_REPAIR: NOT_FUNCTIONING,
    SUSPENDED: NOT_FUNCTIONING,
    NO_CONTINGENT: FUNCTIONING,
    PENDING_OPEN: NOT_FUNCTIONING,
}


def get_dou_status_building(dou, default=None):
    """Код для атрибута `status_building` по "нашему" статусу учреждения."""
    return FUNCTIONING_MAPPING.get(get_dou_status(dou), None or default)


# статусы при которых в случае отсутствия детей в ДОУ
# устанавливаются нулевые свободные места
NOT_CALCULATED_STATUSES = [
    k for k, v in FUNCTIONING_MAPPING.items() if v == NOT_FUNCTIONING
]


def _is_working_dou(dou):
    u"""Проверяет является ли функционирующим ДОУ.

    Функционирующим считается ДОУ у которого (либо у филиалов которого)
    есть дети в плановых / фактических группах.
    """
    result = False

    filials = FilialData.objects.filter(
        head=dou
    ).filter(
        UnitHelper.ALLOWED_FILIAL_FILTER
    ).values_list("filial_id", flat=True).iterator()

    dou_groups = Group.objects.filter(
        status=GroupStatusEnum.FACT
    ).filter(
        Q(unit=dou) |
        Q(unit__in=filials)
    )

    for group in dou_groups:
        children_in_fact = Pupil.objects.filter(
            grup=group, temporary_deduct=False,
            grup__status=GroupStatusEnum.FACT
        )

        children_in_plan = Pupil.objects.filter(
            Q(grup__room_id__isnull=False, grup__room_id=group.room_id)
        ).filter(
            temporary_deduct=False, grup__status=GroupStatusEnum.PLAN
        ).exclude(
            children_id__in=Pupil.objects.filter(
                grup__unit=group.unit, temporary_deduct=False,
                grup__status=GroupStatusEnum.FACT
            ).values_list("children_id", flat=True)
        )

        if children_in_fact.exists() or children_in_plan.exists():
            result = True
            break

    return result


def get_dou_status(dou):
    """Код статуса учреждения в gisdo."""
    if _is_working_dou(dou):
        status = WORK
    elif dou.status == UnitStatus.NONE:
        status = NO_CONTINGENT
    else:
        status = _get_gisdo_unit_status_code(dou.status, default=NO_CONTINGENT)

    return status


def get_dou_status_comment(status, unit):
    """Комментарий к полю статус.

    для status = 1 атрибут не передается,
    для status = 2 и 3 указывается планируемая дата окончания работ,
    для status = 4 и 5 указывается причина и планируемая дата открытия
        (Если статус 5 и при этом наш статус NONE (0) то передаем "нет"),
    для status = 6 указывается планируемая дата открытия.

    :param status: статус учреждения в кодах фед отчетности, см список
    который возвращает get_dou_status_comment
    :param unit: Unit инстанс модели учреждений
    :return: текстовое представление комментария к статусу
    или None если атрибут не надо передавать
    """
    comment = None
    if status in (RECONSTRUCTION, IN_CAPITAL_REPAIR):
        comment = try_date_in_string(unit.end_repair_date)
    elif status == SUSPENDED:
        comment = unit.suspend_missing_cont_reason or ''
        comment += try_date_in_string(unit.commissioning_date)
    elif status == NO_CONTINGENT:
        if unit.status == UnitStatus.NONE:
            comment = 'нет'
        else:
            comment = unit.suspend_missing_cont_reason or ''
            comment += try_date_in_string(unit.commissioning_date)
    elif status == PENDING_OPEN:
        comment = try_date_in_string(unit.build_date)

    return comment


def get_passport(ext_unit):
    """Возвращает ссылку на паспорт доступности
    :param unit:
    :return:
    """
    default = 'нет'
    if not ext_unit:
        return default
    if ext_unit.availability_passport_url:
        return ext_unit.availability_passport_url
    elif ext_unit.availability_passport_doc:
        return get_file_url(ext_unit, "availability_passport_doc")
    return default


def get_dou_structure_type(dou):

    conformity = {
        StructureTypes.KINDER: 1,
        StructureTypes.SCHOOL: 2,
        StructureTypes.PRESOTHER: 3,
        StructureTypes.UNIVERSITY: 4,
    }

    if dou.structure_type == StructureTypes.OTHER:
        return 1
    else:
        return conformity[dou.structure_type]


def get_groups_num(dou):

    filials = FilialData.objects.filter(
        head=dou
    ).filter(
        UnitHelper.ALLOWED_FILIAL_FILTER
    ).values_list("filial_id", flat=True).iterator()

    dou_groups = Group.extended_objects.with_count_norm().select_related(
        'type', 'health_need'
    ).filter(
        status=GroupStatusEnum.FACT
    ).filter(
        Q(unit=dou) | Q(unit__in=filials)
    )

    return dou_groups.count()


def has_fact_groups(dou):
    u"""Проверяет есть ли в ДОУ фактические группы"""

    return Group.objects.filter(
        unit=dou,
        status=GroupStatusEnum.FACT
    ).exists()


def get_dou_filials(dou):
    u"""Возвращает филиалы ДОУ"""

    return FilialData.objects.filter(
        head=dou
    ).filter(UnitHelper.ALLOWED_FILIAL_FILTER)


def get_dou_filials_and_building_ids(dou):
    u"""Возвращает id филиалов и корпусов ДОУ"""
    filials = get_dou_filials(dou).values('filial_id')

    valid_filials = Unit.objects.filter(
        id__in=filials,
        is_filial__in=[FilialType.FILIAL, FilialType.CORPUS])
    return valid_filials.values('id')


def get_filial_num(dou):
    u"""Количество подтвержденных филиалов."""

    filials = get_dou_filials(dou).select_related('filial')

    filials_count = 0
    for filial_data in filials:
        if has_fact_groups(filial_data.filial):
            filials_count += 1

    return filials_count


def get_unit_fias_house_guid(dou):
    u"""Возвращаем фиас код адреса организации."""

    if dou.address_house_guid:
        return dou.address_house_guid
    elif dou.address_street:
        return dou.address_street
    elif dou.address_place:
        return dou.address_place
    else:
        return "00000000-0000-0000-0000-000000000000"


def get_additional_info_by_dou(dou, model=UnitDocFiles):
    u"""Дополнительная информация по организации."""

    filials = UnitHelper.get_filial(dou)
    extensions = model.objects.filter(
        Q(unit=dou) | Q(unit__in=filials)
    ).values_list('name', flat=True)

    if not extensions.exists():
        return u'нет'
    else:
        filled_extensions = [
            extension for extension in extensions
            if extension
        ]
        info = ','.join(filled_extensions)
        if info:
            return info
        else:
            return u'нет'


def _get_ext_unit_value(dou, name_param):
    """Проверяем что значение поля name_param равно True в
    самом доу или его филилале
    :param dou:
    :param name_param: имя парамера
    :return:
    """
    try:
        ext_unit = dou.ext_unit
        value = getattr(ext_unit, name_param)
        if value: return value
        param = dict()
        param['filial__ext_unit__%s' % name_param] = True
        return get_dou_filials(dou).filter(**param).exists()
    except EmieUnitModel.DoesNotExist:
        pass
    return False


def get_num_advisory_centr(dou_list):
    """ Считаем ДОО, у которых выставлен чек бокс "Наличие КЦ в
     ДОО или его филиале
     """
    result = 0
    for dou in dou_list:
        if _get_ext_unit_value(dou, 'counseling_center'):
            result += 1
    return result


def get_num_early_assistance(dou_list):
    """ читаем ДОО, у которых выставлен чек бокс "Наличие КЦ в
     ДОО или его филиале"""
    result = 0
    for dou in dou_list:
        if _get_ext_unit_value(dou, 'early_help_service'):
            result += 1
    return result


def get_filial_info(dou):

    return 1 if UnitHelper.get_filial(dou).exists() else 0


def get_max_doo(mo):
    """Возвращает максимальное количество желаемых ДОО для МО."""
    # максимальное количество ДОО указанное для МО
    if mo.max_desired_dou != -1:
        return mo.max_desired_dou

    # если для МО не указано - максимальное количество для РЕГИОНА
    region = mo.get_ancestors().filter(kind=UnitKind.REGION).first()
    if region is not None and region.max_desired_dou != -1:
        return region.max_desired_dou

    # если у РЕГИОНА не указано - общее количество ДОО в данном муниципалитете
    # (не учитываются Статус учреждения = "Закрытые", "ликвидированные",
    # "присоединненные к другой организации", а так же филиалы и корпуса)
    return mo.get_descendants().filter(
        kind=UnitKind.DOU
    ).exclude(
        status__in=UnitStatus.ALL_CLOSED_STATUS,
    ).exclude(
        is_filial__in=(FilialType.FILIAL, FilialType.CORPUS)
    ).count()


def prepare_url(url, default=None):
    """Подготавливает ссылку url.
    Если url пустой, то вернёт значение по умолчанию (если указано) или сам url.
    Eсли url соответствует паттерну веб адреса и содержит спец символы, то вернёт url обёрнутый в CDATA.
    Если url только соответсвует паттерну веб адреса, то вернёт сам url.
    Если url не соответсвует паттерну веб адреса, то возвращает значение по умолчанию default.
    """
    symbols_to_prepare = ['&']

    if not url:
        return default or url

    # если url соответсвует паттерну веб адреса, то вернуть url
    if website_pattern.match(url) is not None:
        # если в url есть символы, для которых нужно использовать экранирование,
        # то url оборачивается в CDATA
        if any(symbol in url for symbol in symbols_to_prepare):
            return get_prepared_url(url)
        return url
    return default


CDATA_URL_PATTERN = u'<![CDATA[ {} ]]>'


def get_prepared_url(url):
    """Возвращает обёрнутый в CDATA URL"""
    return CDATA_URL_PATTERN.format(url)


def escape_special_symbols(url):
    """экранирует только определённые символы в URL"""
    # символы, что нужно экранировать
    # порядок символов важен, так как после экранирования, например, <,
    # будет дополнительно экранирован его &lt
    escape_symbols_mapping = OrderedDict([
        (u'&', u'&amp'),
        (u'<', u'&lt'),
        (u'>', u'&gt'),
    ])

    if url:
        for symbol in escape_symbols_mapping.keys():
            url = url.replace(symbol, escape_symbols_mapping[symbol])

    return url


def get_employers_in_dou_by_category(dou_ids, category, part_time=False):
    """Возвращает количество неуоленных сотрудников в указанных ДОО
    по категории должности и типу совместительства."""

    query = EmployerPost.objects.filter(
        post__post_category__code__in=category,
        employer__unit_id__in=dou_ids,
        employer__discharged=False
    )
    if not part_time:
        return query.filter(
            Q(part_time=part_time) |
            Q(contingentemployeepost__is_internal_part_time=True)).count()
    return query.filter(contingentemployeepost__is_external_part_time=True
                        ).count()
